from django.utils.encoding import smart_unicode, smart_str, force_unicode
from django import forms
from django.core import validators
from survey import widgets


class ChoiceWithInputField(forms.ChoiceField):

    widget = widgets.RadioSelectWithInput

    def __init__(self, *args, **kwargs):
        self.input_choices = kwargs.pop("input_choices", [])
        super(ChoiceWithInputField, self).__init__(*args, **kwargs)
        self.widget.input_choices = self.input_choices

    def to_python(self, value):
        if type(value) in [list, tuple]:
            value = list(value)
            if value[0] in validators.EMPTY_VALUES:
                value[0] = u''
            if value[1] in validators.EMPTY_VALUES:
                value[1] = u''
            return (smart_unicode(value[0]), smart_unicode(value[1]))
        else:
            if value in validators.EMPTY_VALUES:
                return u''
            return smart_unicode(value)

    def validate(self, value):
        """
        Validates that the input is in self.choices.
        """
        if type(value) in [list, tuple]:
            value = value[0]
        super(ChoiceWithInputField, self).validate(value)
        if value and not self.valid_value(value):
            raise forms.ValidationError(self.error_messages['invalid_choice'] % {'value': value})


class MultipleChoiceWithInputField(forms.MultipleChoiceField):

    widget = widgets.CheckboxSelectMultipleWithInput

    def __init__(self, *args, **kwargs):
        self.input_choices = kwargs.pop("input_choices", [])
        super(MultipleChoiceWithInputField, self).__init__(*args, **kwargs)
        self.widget.input_choices = self.input_choices

    def to_python(self, value):
        if not value:
            return []
        elif not isinstance(value, (list, tuple)):
            raise forms.ValidationError(self.error_messages['invalid_list'])
        return [val if type(val) in (list, tuple) else smart_unicode(val) for val in value]

    def validate(self, value):
        if self.required and not value:
            raise forms.ValidationError(self.error_messages['required'])
        # Validate that each value in the value list is in self.choices.
        for val in value:
            if type(val) in [list, tuple]:
                val = val[0]
            if not self.valid_value(val):
                raise forms.ValidationError(self.error_messages['invalid_choice'] % {'value': val})
